# coding: utf-8
class Const:
    ActionDenyAccess = "deny_access"
    ActionAllowAccess = "allow_access"
    ActionRedirectProtocol = "redirect_protocol"
    ActionRedirectRequest = "redirect_request"
    ActionRequestHeader = "request_header"
    ActionDownloadSpeedLimit = "download_speed_limit"
    ActionResponseHeader = "response_header"
    ActionCacheTime = "cache_time"
    ActionCacheKey = "cache_key"
    ActionCompression = "compression"
    ActionVideoDrag = "video_drag"
    ActionOriginResponseHeader = "origin_response_header"
    ActionOriginRequestHeader = "origin_request_header"
    ActionOriginRange = "origin_range"
    ActionOriginFollow = "origin_follow"
    ActionOriginTCPTimeout = "origin_tcp_timeout"
    ActionOriginHTTPTimeout = "origin_http_timeout"
    ActionOriginHost = "origin_host"
    # ActionVariable = "variable"
    # ActionAccessPlugin = "access_plugin"
    # ActionRewritePlugin = "rewrite_plugin"
    # ActionHeaderFilterPlugin = "header_filter_plugin"
    # ActionBodyFilterPlugin = "body_filter_plugin"
    # ActionLogPlugin = "log_plugin"

    ConditionAlways = "always"
    ConditionRequestTime = "request_time"
    ConditionClientIP = "client_ip"
    ConditionRequestHeader = "request_header"
    ConditionHTTPOrigin = "http_origin"
    ConditionHTTPReferer = "http_referer"
    ConditionHTTPUserAgent = "http_ua"
    ConditionHTTPMethod = "http_method"
    ConditionHTTPProtocol = "protocol"
    ConditionHTTPUrl = "url"
    ConditionHTTPPathAndParam = "path_and_param"
    ConditionHTTPPath = "path"
    ConditionQueryString = "query_string"
    ConditionResponseHeader = "response_header"
    ConditionHTTPCode = "http_code"
    ConditionClientArea = "client_area"

    ConnectiveAnd = "and"
    ConnectiveOr = "or"

    OperatorEqual = "equal"
    OperatorNotEqual = "not_equal"
    OperatorMatch = "match"
    OperatorNotMatch = "not_match"
    OperatorPrefixMatch = "prefix_match"
    OperatorPrefixNotMatch = "prefix_not_match"
    OperatorSuffixMatch = "suffix_match"
    OperatorSuffixNotMatch = "suffix_not_match"
    OperatorExist = "exist"
    OperatorNotExist = "not_exist"
    OperatorRegexMatch = "regex_match"
    OperatorRegexNotMatch = "regex_not_match"
    OperatorBelongTo = "belong_to"
    OperatorNotBelongTo = "not_belong_to"

    EveryDay = "everyday"
    Weekday = "weekday"
    Period = "period"

    Monday = "mon"
    Tuesday = "tue"
    Wednesday = "wed"
    Thursday = "thur"
    Friday = "fri"
    Saturday = "sat"
    Sunday = "sun"

    GET = "get"
    POST = "post"
    HEAD = "head"
    DELETE = "delete"
    OPTIONS = "options"
    PURGE = "purge"
    PUT = "put"
    CONNECT = "connect"
    PATCH = "patch"

    HTTP = "http"
    HTTPS = "https"


    Second = "sec"
    Minute = "min"
    Hour = "hour"
    Day = "day"

    HTTPTimeout = "http_read_timeout"
    TCPTimeout = "tcp_connect_timeout"

    Set = "set"
    Delete = "delete"
    Append = "append"

    SpecificContentType = "customize"
    DefaultContentType = "default"

    QueryString = "query_string"
    RequestPath = "request_path"
    RequestHeader = "request_header"
    Cookie = "cookie"
    PathIgnoreCase = "path_ignore_case"

    FollowClient = "follow_client"

    IncludeAll = "include_all"
    IncludePart = "include_part"
    ExcludeAll = "exclude_all"
    ExcludePart = "exclude_part"

    CacheOriginFirst = "origin_first"
    CacheDefault = "default"

    FluxByte = "B"
    FluxKB = "KB"
    FluxMB = "MB"

    BandwidthBps = "B/s"
    BandwidthKBps = "KB/s"
    BandwidthMBps = "MB/s"

    TimeZoneEast8 = "+0800"

    SwitchTrue = "true"
    SwitchFalse = "false"